#!/usr/bin/env python
# coding: utf-8

# Copyright (c) Manabu TERADA.
# Distributed under the terms of the Modified BSD License.

import pytest
from traitlets.traitlets import TraitError
from ..netvis import NetVis


def test_netvis_creation_blank():
    w = NetVis()
    assert w.value == ""


def test_netvis_creation_with_dict():
    with pytest.raises(TraitError):
        w = NetVis(value={"a": 1})
    # assert isinstance(w.value, str)
    # assert w.value == '{"a": 1}'


def test_netvis_creation_with_list():
    with pytest.raises(TraitError):
        w = NetVis(value=[1, 2, 3])
    # assert isinstance(w.value, str)
    # assert w.value == "[1, 2, 3]"


def test_netvis_creation_with_str():
    w = NetVis(value='{"a": 1}')
    assert isinstance(w.value, str)
    assert w.value == '{"a": 1}'



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) Manabu TERADA.
# Distributed under the terms of the Modified BSD License.

"""
This module defines the NetVis widget.
"""

import json

from ipywidgets import DOMWidget, ValueWidget, register
from traitlets import Unicode, validate, TraitError
from ._frontend import module_name, module_version


def is_invalid_json(data):
    try:
        json.loads(data)
        return False
    except json.JSONDecodeError:
        return True


@register
class NetVis(DOMWidget, ValueWidget):
    """NetVis widget.
    This widget show Network Visualization.
    """

    _model_name = Unicode("NetVisModel").tag(sync=True)
    _model_module = Unicode(module_name).tag(sync=True)
    _model_module_version = Unicode(module_version).tag(sync=True)
    _view_name = Unicode("NetVisView").tag(sync=True)
    _view_module = Unicode(module_name).tag(sync=True)
    _view_module_version = Unicode(module_version).tag(sync=True)

    value = Unicode().tag(sync=True)

    @validate("value")
    def _valid_value(self, proposal):
        # if isinstance(proposal["value"], str):
        #     _data = proposal["value"]
        # elif isinstance(proposal["value"], (dict, list)):
        #     _data = json.dumps(proposal["value"])
        # else:
        #     raise TraitError("Invalid data type: it must be JSON string or dict / list")
        _data = proposal["value"]
        if is_invalid_json(_data):
            raise TraitError("Invalid JSON value: it must be JSON string")
        return _data
